﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Info2Praktikum;

namespace _2011_05_04_Praktikum
{
    class Filter
    {
        public virtual Bitmap Execute(Bitmap input)
        {
            // return input;
            // würde zunächst funktionieren, kann aber für Überraschungen sorgen,
            // weil spätere Änderungen an der ausgegebenen Bitmap ja auf die eingegebene
            // Bitmap durchschlagen. Das ist ja dasselbe Objekt!

            // Sicherer ist:
            Bitmap result = new Bitmap(input.Width, input.Height);
            for (int y = 0; y < result.Height; y++)
            {
                for (int x = 0; x < result.Width; x++)
                {
                    result.SetRed(x, y, input.GetRed(x, y));
                    result.SetGreen(x, y, input.GetGreen(x, y));
                    result.SetBlue(x, y, input.GetBlue(x, y));
                }
            }
            return result;
        }
    }

    class Inverter : Filter
    {
        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(input.Width, input.Height);
            for (int y = 0; y < result.Height; y++)
            {
                for (int x = 0; x < result.Width; x++)
                {
                    result.SetRed(x, y, 255 - input.GetRed(x, y));
                    result.SetGreen(x, y, 255 - input.GetGreen(x, y));
                    result.SetBlue(x, y, 255 - input.GetBlue(x, y));
                }
            }
            return result;
        }

        public override string ToString()
        {
            return "Farben umkehren";
        }
    }

    class HorizontalFlipper : Filter
    {
        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(input.Width, input.Height);
            for (int y = 0; y < result.Height; y++)
            {
                for (int x = 0; x < result.Width; x++)
                {
                    int x1 = result.Width - 1 - x;
                    result.SetRed(x1, y, input.GetRed(x, y));
                    result.SetGreen(x1, y, input.GetGreen(x, y));
                    result.SetBlue(x1, y, input.GetBlue(x, y));
                }
            }
            return result;
        }

        public override string ToString()
        {
            return "Horizontal spiegeln";
        }
    }

    class BrightnessAdjuster : Filter
    {
        double brightnessFactor = 1.0;

        public BrightnessAdjuster(double brightnessFactor)
        {
            this.brightnessFactor = brightnessFactor;
        }

        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(input.Width, input.Height);
            for (int y = 0; y < result.Height; y++)
            {
                for (int x = 0; x < result.Width; x++)
                {
                    result.SetRed(x, y, (int)Math.Min(255, brightnessFactor * input.GetRed(x, y)));
                    result.SetGreen(x, y, (int)Math.Min(255, brightnessFactor * input.GetGreen(x, y)));
                    result.SetBlue(x, y, (int)Math.Min(255, brightnessFactor * input.GetBlue(x, y)));
                }
            }
            return result;
        }

        public override string ToString()
        {
            if (brightnessFactor >= 1.0)
            {
                return "Aufhellen um " + 100.0*(brightnessFactor-1.0) + " Prozent";
            }
            else
            {
                return "Abdunkeln um " + 100.0*(1.0-brightnessFactor) + " Prozent";
            }
        }
    }

    class Cropper : Filter
    {
        int x, y, width, height;

        public Cropper(int x, int y, int width, int height)
        {
            this.x = x;
            this.y = y;
            this.width = width;
            this.height = height;
        }

        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(width, height);
            for (int y1 = 0; y1 < height; y1++)
            {
                // Math.Min, damit nicht ggf. über Rand hinaus
                int y2 = Math.Min(y1, input.Height-1);
                for (int x1 = 0; x1 < width; x1++)
                {
                    int x2 = Math.Min(x1, input.Width-1);
                    result.SetRed(x1, y1, input.GetRed(x2, y2));
                    result.SetGreen(x1, y1, input.GetGreen(x2, y2));
                    result.SetBlue(x1, y1, input.GetBlue(x2, y2));
                }
            }
            return result;
        }

        public override string ToString()
        {
            return "Bildausschnitt " + x +", " + y + ", " + (x+width) + ", " + (y+height);
        }
    }
}
